@@ -1,6 +1,5 @@
 # coding: utf-8
 
-from __future__ import division, print_function, unicode_literals, absolute_import
 
 import os
 import datetime
@@ -13,7 +12,7 @@ from itertools import chain
 
 from monty.io import zopen
 from monty.json import jsanitize
-from pymatgen.io.qchem.outputs import QCOutput
+from pymatgen.io.qchem.outputs import QCOutput, check_for_structure_changes
 from pymatgen.io.qchem.inputs import QCInput
 from pymatgen.apps.borg.hive import AbstractDrone
 from pymatgen.io.babel import BabelMolAdaptor
@@ -46,10 +45,10 @@ class QChemDrone(AbstractDrone):
         "root": {
             "dir_name", "input", "output", "calcs_reversed", "smiles",
             "walltime", "cputime", "formula_pretty", "formula_anonymous",
-            "chemsys", "pointgroup"
+            "chemsys", "pointgroup", "formula_alphabetical"
         },
         "input": {"initial_molecule", "job_type"},
-        "output": {"initial_molecule", "job_type"}
+        "output": {"initial_molecule", "job_type", "final_energy"}
     }
 
     def __init__(self, runs=None, additional_fields=None):
@@ -168,6 +167,18 @@ class QChemDrone(AbstractDrone):
             # reverse the calculations data order so newest calc is first
             d["calcs_reversed"].reverse()
 
+            d["structure_change"] = []
+            d["warnings"] = {}
+            for entry in d["calcs_reversed"]:
+                if "structure_change" in entry and "structure_change" not in d["warnings"]:
+                    if entry["structure_change"] != "no_change":
+                        d["warnings"]["structure_change"] = True
+                if "structure_change" in entry:
+                    d["structure_change"].append(entry["structure_change"])
+                for key in entry["warnings"]:
+                    if key not in d["warnings"]:
+                        d["warnings"][key] = True
+
             d_calc_init = d["calcs_reversed"][-1]
             d_calc_final = d["calcs_reversed"][0]
 
@@ -177,8 +188,13 @@ class QChemDrone(AbstractDrone):
             }
             d["output"] = {
                 "initial_molecule": d_calc_final["initial_molecule"],
-                "job_type": d_calc_final["input"]["rem"]["job_type"]
+                "job_type": d_calc_final["input"]["rem"]["job_type"],
+                "mulliken": d_calc_final["Mulliken"][-1]
             }
+            if "RESP" in d_calc_final:
+                d["output"]["resp"] = d_calc_final["RESP"][-1]
+            elif "ESP" in d_calc_final:
+                d["output"]["esp"] = d_calc_final["ESP"][-1]
 
             if d["output"]["job_type"] == "opt" or d["output"]["job_type"] == "optimization":
                 if "molecule_from_optimized_geometry" in d_calc_final:
@@ -202,8 +218,13 @@ class QChemDrone(AbstractDrone):
                     d["output"]["final_energy"] = d["calcs_reversed"][1][
                         "final_energy"]
 
-            if d["output"]["job_type"] == "sp":
-                d["output"]["final_energy"] = d_calc_final["final_energy"]
+            if "final_energy" not in d["output"]:
+                if d_calc_final["final_energy"] != None:
+                    d["output"]["final_energy"] = d_calc_final["final_energy"]
+                else:
+                    d["output"]["final_energy"] = d_calc_final["SCF"][-1][-1][0]
+                # else:
+                #     print(d_calc_final)
 
             if d_calc_final["completion"]:
                 total_cputime = 0.0
@@ -222,6 +243,7 @@ class QChemDrone(AbstractDrone):
             comp = d["output"]["initial_molecule"].composition
             d["formula_pretty"] = comp.reduced_formula
             d["formula_anonymous"] = comp.anonymized_formula
+            d["formula_alphabetical"] = comp.alphabetical_formula
             d["chemsys"] = "-".join(sorted(set(d_calc_final["species"])))
             if d_calc_final["point_group"] != None:
                 d["pointgroup"] = d_calc_final["point_group"]
@@ -239,10 +261,57 @@ class QChemDrone(AbstractDrone):
             d["state"] = "successful" if d_calc_final["completion"] else "unsuccessful"
             if "special_run_type" in d:
                 if d["special_run_type"] == "frequency_flattener":
-                    d["num_frequencies_flattened"] = int((len(qcinput_files) / 2) - 1)
+                    opt_traj = []
+                    for entry in d["calcs_reversed"]:
+                        if entry["input"]["rem"]["job_type"] == "opt" or entry["input"]["rem"]["job_type"] == "optimization":
+                            doc = {"initial": {}, "final": {}}
+                            doc["initial"]["molecule"] = entry["initial_molecule"]
+                            doc["final"]["molecule"] = entry["molecule_from_last_geometry"]
+                            doc["initial"]["total_energy"] = entry["energy_trajectory"][0]
+                            doc["final"]["total_energy"] = entry["energy_trajectory"][-1]
+                            doc["initial"]["scf_energy"] = entry["SCF"][0][-1][0]
+                            doc["final"]["scf_energy"] = entry["SCF"][-1][-1][0]
+                            doc["structure_change"] = entry["structure_change"]
+                            opt_traj.append(doc)
+                    opt_traj.reverse()
+                    opt_trajectory = {"trajectory": opt_traj, "structure_change": [[ii, entry["structure_change"]] for ii,entry in enumerate(opt_traj)], "energy_increase": []}
+                    for ii, entry in enumerate(opt_traj):
+                        if entry["final"]["total_energy"] > entry["initial"]["total_energy"]:
+                            opt_trajectory["energy_increase"].append([ii, entry["final"]["total_energy"]-entry["initial"]["total_energy"]])
+                        if ii != 0:
+                            if entry["final"]["total_energy"] > opt_traj[ii-1]["final"]["total_energy"]:
+                                opt_trajectory["energy_increase"].append([ii-1, ii, entry["final"]["total_energy"]-opt_traj[ii-1]["final"]["total_energy"]])
+                            struct_change = check_for_structure_changes(opt_traj[ii-1]["final"]["molecule"], entry["final"]["molecule"])
+                            if struct_change != entry["structure_change"]:
+                                opt_trajectory["structure_change"].append([ii-1, ii, struct_change])
+                                d["warnings"]["between_iteration_structure_change"] = True
+                    if "linked" in d:
+                        if d["linked"] == True:
+                            opt_trajectory["discontinuity"] = {"structure": [], "scf_energy": [], "total_energy": []}
+                            for ii, entry in enumerate(opt_traj):
+                                if ii != 0:
+                                    if entry["initial"]["molecule"] != opt_traj[ii-1]["final"]["molecule"]:
+                                        opt_trajectory["discontinuity"]["structure"].append([ii-1,ii])
+                                        d["warnings"]["linked_structure_discontinuity"] = True
+                                    if entry["initial"]["total_energy"] != opt_traj[ii-1]["final"]["total_energy"]:
+                                        opt_trajectory["discontinuity"]["total_energy"].append([ii-1,ii])
+                                    if entry["initial"]["scf_energy"] != opt_traj[ii-1]["final"]["scf_energy"]:
+                                        opt_trajectory["discontinuity"]["scf_energy"].append([ii-1,ii])
+                    d["opt_trajectory"] = opt_trajectory
+
                     if d["state"] == "successful":
-                        if d_calc_final["frequencies"][0] < 0: # If a negative frequency remains,
-                            d["state"] = "unsuccessful" # then the flattening was unsuccessful
+                        orig_num_neg_freq = sum(1 for freq in d["calcs_reversed"][-2]["frequencies"] if freq < 0)
+                        orig_energy = d_calc_init["final_energy"]
+                        final_num_neg_freq = sum(1 for freq in d_calc_final["frequencies"] if freq < 0)
+                        final_energy = d["calcs_reversed"][1]["final_energy"]
+                        d["num_frequencies_flattened"] = orig_num_neg_freq - final_num_neg_freq
+                        if final_num_neg_freq > 0: # If a negative frequency remains,
+                            # and it's too large to ignore,
+                            if final_num_neg_freq > 1 or abs(d["output"]["frequencies"][0]) >= 15.0:
+                                d["state"] = "unsuccessful" # then the flattening was unsuccessful
+                        if final_energy > orig_energy:
+                            d["warnings"]["energy_increased"] = True
+
             d["last_updated"] = datetime.datetime.utcnow()
             return d
 
@@ -307,7 +376,6 @@ class QChemDrone(AbstractDrone):
     def post_process(dir_name, d):
         """
         Post-processing for various files other than the QChem input and output files.
-        Currently only looks for custodian.json. Modify this if other files need to be processed.
         """
         logger.info("Post-processing dir:{}".format(dir_name))
         fullpath = os.path.abspath(dir_name)
@@ -315,6 +383,10 @@ class QChemDrone(AbstractDrone):
         if len(filenames) >= 1:
             with zopen(filenames[0], "rt") as f:
                 d["custodian"] = json.load(f)
+        filenames = glob.glob(os.path.join(fullpath, "solvent_data*"))
+        if len(filenames) >= 1:
+            with zopen(filenames[0], "rt") as f:
+                d["custom_smd"] = f.readlines()[0]
 
     def validate_doc(self, d):
         """
@@ -324,7 +396,7 @@ class QChemDrone(AbstractDrone):
         for k, v in self.schema.items():
             diff = v.difference(set(d.get(k, d).keys()))
             if diff:
-                logger.warn("The keys {0} in {1} not set".format(diff, k))
+                logger.warning("The keys {0} in {1} not set".format(diff, k))
 
     @staticmethod
     def get_valid_paths(self, path):
