@@ -1,15 +1,12 @@
 # coding: utf-8
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import six
 from abc import ABCMeta, abstractmethod
 
 __author__ = "Kiran Mathew"
 __email__ = "kmathew@lbl.gov"
 
 
-class AbstractBuilder(six.with_metaclass(ABCMeta)):
+class AbstractBuilder(metaclass=ABCMeta):
     """
     Abstract builder class. Defines the contract and must be subclassed by all builders.
     """
