@@ -1,6 +1,5 @@
 # coding: utf-8
 
-from __future__ import division, print_function, unicode_literals, absolute_import
 
 from monty.json import MontyEncoder, MontyDecoder
 
@@ -34,8 +33,9 @@ class VaspCalcDb(CalcDb):
     """
 
     def __init__(self, host="localhost", port=27017, database="vasp", collection="tasks", user=None,
-                 password=None):
-        super(VaspCalcDb, self).__init__(host, port, database, collection, user, password)
+                 password=None, **kwargs):
+        super(VaspCalcDb, self).__init__(host, port, database, collection, user,
+                                         password, **kwargs)
 
     def build_indexes(self, indexes=None, background=True):
         """
@@ -70,7 +70,9 @@ class VaspCalcDb(CalcDb):
     def insert_task(self, task_doc, use_gridfs=False):
         """
         Inserts a task document (e.g., as returned by Drone.assimilate()) into the database.
-        Handles putting DOS and band structure into GridFS as needed.
+        Handles putting DOS, band structure and charge density into GridFS as needed.
+        During testing, a percentage of runs on some clusters had corrupted AECCAR files when even if everything else about the calculation looked OK.
+        So we do a quick check here and only record the AECCARs if they are valid
 
         Args:
             task_doc: (dict) the task document
@@ -80,8 +82,7 @@ class VaspCalcDb(CalcDb):
         """
         dos = None
         bs = None
-        chgcar = None
-        aeccar0 = None
+        vol_data = {}
 
         # move dos BS and CHGCAR from doc to gridfs
         if use_gridfs and "calcs_reversed" in task_doc:
@@ -94,20 +95,11 @@ class VaspCalcDb(CalcDb):
                 bs = json.dumps(task_doc["calcs_reversed"][0]["bandstructure"], cls=MontyEncoder)
                 del task_doc["calcs_reversed"][0]["bandstructure"]
 
-            if "chgcar" in task_doc["calcs_reversed"][0]:  # only store idx=0 DOS
-                chgcar = json.dumps(task_doc["calcs_reversed"][0]["chgcar"], cls=MontyEncoder)
-                del task_doc["calcs_reversed"][0]["chgcar"]
-
-            if "aeccar0" in task_doc["calcs_reversed"][0]:  # only store idx=0 DOS
-                aeccar0 = task_doc["calcs_reversed"][0]["aeccar0"]
-                aeccar0 = json.dumps(task_doc["calcs_reversed"][0]["aeccar0"], cls=MontyEncoder)
-                del task_doc["calcs_reversed"][0]["aeccar0"]
-                try:
-                    # aeccar2 should also be in the task_doc
-                    aeccar2 = json.dumps(task_doc["calcs_reversed"][0]["aeccar2"], cls=MontyEncoder)
-                except:
-                    raise KeyError('aeccar2 data is missing from task_doc')
-                del task_doc["calcs_reversed"][0]["aeccar2"]
+            for vol_data_name in ('chgcar', 'locpot', 'aeccar0', 'aeccar1', 'aeccar2', 'elfcar'):
+                if vol_data_name in task_doc["calcs_reversed"][0]:  # only store idx=0 data
+                    vol_data[vol_data_name] = json.dumps(task_doc["calcs_reversed"][0][vol_data_name],
+                                                         cls=MontyEncoder)
+                    del task_doc["calcs_reversed"][0][vol_data_name]
 
         # insert the task document
         t_id = self.insert(task_doc)
@@ -128,22 +120,13 @@ class VaspCalcDb(CalcDb):
                 {"task_id": t_id}, {"$set": {"calcs_reversed.0.bandstructure_fs_id": bfs_gfs_id}})
 
         # insert the CHGCAR file into gridfs and update the task documents
-        if chgcar:
-            chgcar_gfs_id, compression_type = self.insert_gridfs(chgcar, "chgcar_fs", task_id=t_id)
-            self.collection.update_one(
-                {"task_id": t_id}, {"$set": {"calcs_reversed.0.chgcar_compression": compression_type}})
-            self.collection.update_one({"task_id": t_id}, {"$set": {"calcs_reversed.0.chgcar_fs_id": chgcar_gfs_id}})
+        if vol_data:
+            for name, data in vol_data.items():
+                data_gfs_id, compression_type = self.insert_gridfs(data, "{}_fs".format(name), task_id=t_id)
+                self.collection.update_one(
+                    {"task_id": t_id}, {"$set": {"calcs_reversed.0.{}_compression".format(name): compression_type}})
+                self.collection.update_one({"task_id": t_id}, {"$set": {"calcs_reversed.0.{}_fs_id".format(name): data_gfs_id}})
 
-        # insert the AECCARs file into gridfs and update the task documents
-        if aeccar0:
-            aeccar0_gfs_id, compression_type = self.insert_gridfs(aeccar0, "aeccar0_fs", task_id=t_id)
-            self.collection.update_one(
-                {"task_id": t_id}, {"$set": {"calcs_reversed.0.aeccar0_compression": compression_type}})
-            self.collection.update_one({"task_id": t_id}, {"$set": {"calcs_reversed.0.aeccar0_fs_id": aeccar0_gfs_id}})
-            aeccar2_gfs_id, compression_type = self.insert_gridfs(aeccar2, "aeccar2_fs", task_id=t_id)
-            self.collection.update_one(
-                {"task_id": t_id}, {"$set": {"calcs_reversed.0.aeccar2_compression": compression_type}})
-            self.collection.update_one({"task_id": t_id}, {"$set": {"calcs_reversed.0.aeccar2_fs_id": aeccar2_gfs_id}})
         return t_id
 
     def retrieve_task(self, task_id):
@@ -168,6 +151,10 @@ class VaspCalcDb(CalcDb):
         if 'chgcar_fs_id' in calc:
             chgcar = self.get_chgcar(task_id)
             calc["chgcar"] = chgcar
+        if 'aeccar0_fs_id' in calc:
+            aeccar = self.get_aeccar(task_id)
+            calc["aeccar0"] = aeccar['aeccar0']
+            calc["aeccar2"] = aeccar['aeccar2']
         return task_doc
 
     def insert_gridfs(self, d, collection="fs", compress=True, oid=None, task_id=None):
@@ -243,13 +230,14 @@ class VaspCalcDb(CalcDb):
         chgcar= json.loads(chgcar_json, cls=MontyDecoder)
         return chgcar
 
-    def get_aeccar(self, task_id):
+    def get_aeccar(self, task_id, check_valid = True):
         """
         Read the AECCAR0 + AECCAR2 grid_fs data into a Chgcar object
         Args:
             task_id(int or str): the task_id containing the gridfs metadata
+            check_valid (bool): make sure that the aeccar is positive definite
         Returns:
-            (aeccar0, aeccar2): Chgcar objects
+            {"aeccar0" : Chgcar, "aeccar2" : Chgcar}: dict of Chgcar objects
         """
         m_task = self.collection.find_one({"task_id": task_id}, {"calcs_reversed": 1})
         fs_id = m_task['calcs_reversed'][0]['aeccar0_fs_id']
@@ -261,6 +249,9 @@ class VaspCalcDb(CalcDb):
         aeccar_json = zlib.decompress(fs.get(fs_id).read())
         aeccar2 = json.loads(aeccar_json, cls=MontyDecoder)
 
+        if check_valid and (aeccar0.data['total'] + aeccar2.data['total']).min() < 0:
+            ValueError(f"The AECCAR seems to be corrupted for task_id = {task_id}")
+
         return {'aeccar0': aeccar0, 'aeccar2': aeccar2}
 
     def reset(self):
